import 'package:hive/hive.dart';
import 'package:live_repository/src/operation/operations.dart';

class HiveStoreOperation<TEntity> extends StoreOperation<TEntity> {

  Box<TEntity> _box;

  HiveStoreOperation(this._box, TEntity entity, dynamic Function(TEntity entity) keySelector) : super(keySelector, entity);

  @override
  Future<bool> executeAsync() async {
    try {
      await _box.put(keySelector(entity), entity);
      return true;
    } catch (e) {
      return false;
    }
  }
}

class HiveBatchStoreOperation<TEntity> extends BatchStoreOperation<TEntity> {

  Box<TEntity> _box;

  HiveBatchStoreOperation(this._box, List<TEntity> entities, dynamic Function(TEntity entity) keySelector) : super(keySelector, entities);

  @override
  Future<bool> executeAsync() async {
    try {
      for(TEntity entity in entities) {
        await _box.put(keySelector(entity), entity);
      }
      return true;
    } catch(e) {
      return false;
    }
  }

}

class HiveDeleteOperation<TEntity> extends DeleteOperation<TEntity> {

  Box<TEntity> _box;

  HiveDeleteOperation(this._box, TEntity entity, dynamic Function(TEntity entity) keySelector) : super(keySelector, entity);

  @override
  Future<bool> executeAsync() async {
    try {
      await _box.delete(keySelector(entity));
      return true;
    } catch(e) {
      return false;
    }
  }

}

class HiveUpdateOperation<TEntity> extends UpdateOperation<TEntity> {

  Box<TEntity> _box;

  HiveUpdateOperation(this._box, TEntity entity, dynamic Function(TEntity entity) keySelector) : super(keySelector, entity);

  @override
  Future<bool> executeAsync() async {
    if(!_box.containsKey(keySelector(entity))) {
      return false;
    }

    try {
      await _box.put(keySelector(entity), entity);
      return true;
    } catch(e) {
      return true;
    }
  }

}