import 'package:live_repository/src/operation/base_operation.dart';
import 'package:meta/meta.dart';

/// Stores an entity in a repository
abstract class StoreOperation<TEntity> implements BaseOperation {
  
  @protected
  TEntity entity;

  @protected
  dynamic Function(TEntity entity) keySelector;

  StoreOperation(this.keySelector, this.entity);

  @override
  Future<bool> executeAsync();

}

/// Stores many entities in a repository
abstract class BatchStoreOperation<TEntity> implements BaseOperation {
  @protected
  List<TEntity> entities;

  @protected
  dynamic Function(TEntity entity) keySelector;

  BatchStoreOperation(this.keySelector, this.entities);

  @override
  Future<bool> executeAsync();
}

/// Deletes an entity from a repository
abstract class DeleteOperation<TEntity> implements BaseOperation {
  
  @protected
  TEntity entity;

  @protected
  dynamic Function(TEntity entity) keySelector;
  
  DeleteOperation(this.keySelector, this.entity);

  @override
  Future<bool> executeAsync();
}

/// Updates an entity in a repository
abstract class UpdateOperation<TEntity> implements BaseOperation {
  @protected
  TEntity entity;

  @protected
  dynamic Function(TEntity entity) keySelector;

  UpdateOperation(this.keySelector, this.entity);

  @override
  Future<bool>executeAsync();
}