import 'package:hive/hive.dart';
import 'package:live_repository/src/operation/hive_operations.dart';
import 'package:live_repository/src/operation/operations.dart';
import 'package:live_repository/src/repository/repository.dart';
import 'package:live_repository/src/repository/repository_options.dart';

class HiveRepository<TEntity, TKey> extends BaseRepository<TEntity, TKey> {
  
  Box<TEntity> _box;

  @override
  List<TEntity> get entities => _box.values.toList().cast<TEntity>();

  HiveRepository(String name, RepositoryOptions options, this._box, TKey Function(TEntity entity) keySelector) : super(repositoryName: name, options: options, keySelector: keySelector) {
    if(!_box.isOpen) {
      throw Exception("Box ${_box.name} needs to be opened.");
    }
  }

  @override
  BatchStoreOperation<TEntity> createBatchStoreOperation(Map<TKey, TEntity> entities) {
    return HiveBatchStoreOperation<TEntity>(_box, entities.values, keySelector);
  }

  @override
  DeleteOperation<TEntity> createDeleteOperation(TKey key, TEntity value) {
    return HiveDeleteOperation<TEntity>(_box, value, keySelector);
  }

  @override
  StoreOperation<TEntity> createStoreOperation(TKey key, TEntity value) {
    return HiveStoreOperation<TEntity>(_box, value, keySelector);
  }

  @override
  UpdateOperation<TEntity> createUpdateOperation(TKey key, TEntity updatedValue) {
    return HiveUpdateOperation<TEntity>(_box, updatedValue, keySelector);
  }

  @override
  bool existsWrapper(TKey key) {
    return _box.containsKey(key);
  }

  @override
  TEntity findWrapper(TKey key) {
    return _box.get(key);
  }
  
}