/// Contains the options for a repository
class RepositoryOptions {

  /// If true, when an entity is not found in a repository, an [EntityNotFoundException] will be
  /// thrown, otherwise, null is returned.
  /// Default value: [false]
  bool throwExceptionOnNotFound = false;

  /// If true, when an entity already exists in a repository, an [DuplicatedEntityException] will be
  /// thrown, otherwise, its ignored.
  /// Default value: [false]
  bool throwExceptionIfDuplicated = false;

  /// If true, set [updateStampFunction] to a property that returns the [updateStamp] used to
  /// verify and check for changes in a remote server.
  /// Default value: [false]
  bool useUpdateStamp = false;

  /// The function that must return a [String] containing the [updateStamp] value 
  String Function(dynamic entity) updateStampFunction;

  RepositoryOptions({this.throwExceptionIfDuplicated, this.throwExceptionOnNotFound});
}